<?php
namespace FLXDashboardGroups;

class SettingsModule{

    private static $prefix = 'flxdg-';
    private static $resetConfirmMsg =
        '<p style="display:block;padding:1em;background:#fff;">
            ✔ Settings deleted
        </p>';

    private static $resetConfirmForm =
        '<form method="POST"><p style="display:block;padding:1em;background:#fff;">
            ⚠️ Are you sure ? <em>Plugin settings will be deleted</em>
            <br><br>
            <input class="button button-primary" type="submit" name="flxdg-resetSettingsConfirmed" value="Yes"/>
            <a href="?page=flx_dashboard_groups&tab=settings" class="button" >No</a>
        </p></form>';

    private static $fields = [
        'enable_plugins_groups' => ['title' => 'Enable plugins groups', 'default' => 1],
        'enable_pages_groups' => ['title' => 'Enable pages groups', 'default' => 1],
        'enable_medias_groups' => ['title' => 'Enable medias groups', 'default' => 1],
        'display_unclassified' => ['title' => 'Display unclassified', 'default' => 1],
        'hide_empty_groups' => ['title' => 'Hide empty groups', 'default' => 0],
    ];
    
    public static $settings;
    public static $data;
    private static $data_meta = [
        'PLUGIN_GROUPS_OPTION' => ['type' => 'option'],
        'PLUGIN_ASSIGNED_GROUPS_OPTION' => ['type' => 'option'],
        'PAGE_GROUPS_OPTION' => ['type' => 'option'],
        'PAGE_GROUP_META' => ['type' => 'postmeta'],
        'SETTINGS_OPTION' => ['type' => 'option'],
    ];

    const SETTINGS_OPTION = 'flxdg_settings';

    const PLUGIN_GROUPS_OPTION = 'flxdg_plugin_groups';
    const PLUGIN_ASSIGNED_GROUPS_OPTION = 'flxdg_plugin_assigned_groups';

    const PAGE_GROUPS_OPTION = 'flxdg_page_groups';
    const PAGE_GROUP_META = 'flxdg_page_group';
    
    function __construct(){
        global $wpdb;
        
        if(isset($_POST['flxdg-resetSettingsConfirmed'])){
            $this->resetSettings();
        }

        if(isset($_POST[self::$prefix.'save-settings'])){
            $new_settings = isset($_POST[self::$prefix.'settings'])
                ? $_POST[self::$prefix.'settings'] : [];
            update_option(self::SETTINGS_OPTION, $new_settings);
            self::$settings = $new_settings;
        }
        
        self::$settings = get_option(self::SETTINGS_OPTION, []) ?: [];
        

        self::$data = [
            'PLUGIN_GROUPS_OPTION' => get_option(self::PLUGIN_GROUPS_OPTION, []) ?: [],
            'PLUGIN_ASSIGNED_GROUPS_OPTION' => get_option(self::PLUGIN_ASSIGNED_GROUPS_OPTION, []) ?: [],
            'PAGE_GROUPS_OPTION' => get_option(self::PAGE_GROUPS_OPTION, []) ?: [],
            'PAGE_GROUP_META' => $wpdb->get_results( 
                "SELECT pm.post_id as page, pm.meta_value as page_group
                FROM {$wpdb->postmeta} pm
                WHERE pm.meta_key = '".self::PAGE_GROUP_META."'",
                OBJECT_K
            ),
            'SETTINGS_OPTION' => self::$settings
        ];
    }
    
    public function display(){
        global $wpdb;
        $debug_view = isset($_GET['debug_view']);
        ?>
        <form method="POST">
            <h2>Settings</h2>
            <?php
                if(isset($_POST['flxdg-resetSettingsConfirmed'])) echo self::$resetConfirmMsg;
                if(isset($_POST['flxdg-resetSettings'])) echo self::$resetConfirmForm;
            ?>
            <table class="wp-list-table widefat striped fixed">
                <?php
                    foreach(self::$fields as $key => $field){
                        $checked = (
                            (isset(self::$settings[$key]) && (self::$settings[$key] == 1)) 
                            || (!isset(self::$settings[$key]) && ($field['default'] == 1))
                        ) ? ' checked' : '';
                        echo
                            '<tr>
                                <td>
                                    <label for="'.self::$prefix.$key.'">'.$field['title'].'</label>
                                </td>
                                <td>
                                    <input name="'.self::$prefix.'settings['.$key.']" type="hidden" value="0"'.'/>
                                    <input name="'.self::$prefix.'settings['.$key.']" id="'.self::$prefix.$key.'"  type="checkbox" value="1"'.$checked.'/>
                                </td>
                            </tr>';
                    }
                ?>
                <tr>
                    <td>
                        <br>
                        <input name="<?php echo self::$prefix; ?>save-settings" class="button button-primary" type="submit" value="Save settings"/>
                    </td>
                    <td></td>
                </tr>
            </table>
            <br>
        </form>
        <form method="POST">
            <fieldset disabled style="opacity:0.6;">
            <h3>Import / Export (soon)</h3>
                <table class="wp-list-table widefat striped">
                    <thead>
                        <tr>
                            <td>Backup plugin settings</td>
                            <td></td>
                            <td>
                                <button class="button button-primary">Export</button>
                            </td>
                        </tr>
                        <tr>
                            <td>Import plugin settings from backup</td>
                            <td><input type="file"/></td>
                            <td>
                                <button class="button button-primary">Import</button>
                            </td>
                        </tr>
                    </thead>
                </table>
            </fieldset>
        </form>
        <form method="POST">
            <h3>Plugin data</h3>
            <button type="submit" class="button" name="flxdg-resetSettings">⚠️ Reset plugin (delete all data saved by this plugin)</button>
            <table class="wp-list-table widefat striped">
                <thead>
                    <tr>
                        <th>Name</th>
                        <th>Type</th>
                        <?php if($debug_view){ ?><th>Data</th><?php } ?>
                    </tr>
                </thead>
                <?php
                    foreach(self::$data as $key => $data){
                        echo '<tr>
                            <td><b>'.constant('self::'. $key).'</b></td>
                            <td>'.self::$data_meta[$key]['type'].'</td>
                            <td>'.((!$debug_view) ? ''
                                    : '<pre style="white-space:pre-line;">'
                                        .((!empty(self::$data[$key]))
                                            ? print_r($data, true)
                                            : '<em>No data</em>')
                                    .'</pre>'
                                )
                            .'</td>
                        </tr>';
                    }
                ?>
                <tr>
                    <td colspan=2>
                        <a href="?page=flx_dashboard_groups&tab=settings&debug_view" class="button">🔍 Debug view</a>
                    </td>
                </tr>
            </table>
        </form>
        <?php
    }

    private function resetSettings(){
        global $wpdb;
        foreach(self::$data_meta as $key => $data){
            switch($data['type']){
                default: break;
                case 'option':
                    delete_option(constant('self::'. $key));
                    break;
                case 'postmeta':
                    $wpdb->query(
                        "DELETE FROM {$wpdb->postmeta} 
                        WHERE meta_key = '".constant('self::'. $key)."'"
                    );
                    break;
            }
        }
    }
}